@@ -1,4 +1,4 @@
-push our @foo, "bar was here";
+#push our @foo, "bar was here";
 
 set_stdout(
     "bar done\n".story_var('VAR')."\n"
